@@ -359,11 +359,9 @@ return((sum(acc)/N)*100)
 #' @param K **optional** Scale factor determining how much the WElo and Elo rates change over time. Valid choices are:
 #' "Kovalchik" (by default), "Grand_Slam", "Surface_Hard", "Surface_Grass", "Surface_Clay" and, finally, a constant value \eqn{K}.
 #' The first option ("Kovalchik") is equal to what was suggested by \insertCite{kovalchik_2016;textual}{welo}, 
-#' Putting \eqn{K} to "Grand_Slam" lets the scale factor equal to the first option, with the difference that each Grand Slam match
-#' is multiplied by 1.1. Similarly, the choices "Surface_Hard", "Surface_Grass" and "Surface_Clay" make the scale factor
-#' equal to the first option but, in case of a match disputed on hard, grass or clay, respectively, the scale factor 
-#' increases by 1.1. Finally, it can be any scalar value \eqn{K}. In this case, the scale factor is constant,
-#' indipendently of the number of matches played before the match \eqn{t}
+#' Putting \eqn{K} to "Grand_Slam" lets the Kovalchik scale factor multiplied by 1.1, if the match is a Grand Slam match. 
+#' Similarly, the choices "Surface_Hard", "Surface_Grass" and "Surface_Clay" make the Kovalchik scale factor
+#' increased by 1.1 if, respectively, the match is played on hard, grass or clay. Finally, \eqn{K} can be any scalar value, indipendently of the number of matches played before the match \eqn{t}
 #' @param CI **optional** Confidence intervals for the WElo and Elo rates. Default to FALSE. If 'CI' is set to "TRUE", then the 
 #' confidence intervals are calculated, according to the procedure explained by \insertCite{angelini2021weighted;textual}{welo}
 #' @param alpha **optional** Significance level of the confidence interval. Default to 0.05
@@ -372,8 +370,9 @@ return((sum(acc)/N)*100)
 #' in the parameter 'x')
 #' @return \code{welofit} returns an list containing the following components:
 #' \itemize{
-#' 	\item results: The data.frame including a variety of variables, among which there are the estimated WElo and Elo rates ('WElo_i', 'WElo_j', 'Elo_i' and 'Elo_j', respectively),
-#'  their lower and upper confidence intervals (if CI=TRUE), labelled as '_lb' and '_ub', respectively, and the probability of winning the match for player \eqn{i} (labelled as 'WElo_pi_hat' and 
+#' 	\item results: The data.frame including a variety of variables, among which there are the estimated WElo and Elo rates, before and 
+#' after the match \eqn{t}, for players \eqn{i} and \eqn{j},
+#'  the lower and upper confidence intervals (if CI=TRUE) for the WElo and Elo rates, labelled as '_lb' and '_ub', respectively, and the probability of winning the match for player \eqn{i} (labelled as 'WElo_pi_hat' and 
 #' 'Elo_pi_hat', respectively, for the WElo and Elo models). 
 #'   \item matches: The number of matches analyzed.
 #'   \item period: The sample period considered.
@@ -391,6 +390,11 @@ return((sum(acc)/N)*100)
 #' data(atp_2019) 
 #' db_clean<-clean(atp_2019)
 #' res<-welofit(db_clean)
+#' # append new data
+#' db_clean_1<-db_clean[1:500,]
+#' db_clean_2<-db_clean[501:1200,]
+#' res_1<-welofit(db_clean_1)
+#' res_2<-welofit(res_1,new_data=db_clean_2)
 #' }
 #' @export
 
@@ -421,31 +425,35 @@ TT<-nrow(x)
 
 ################################# Standard Elo
 
-x$Elo_i<-rep(SP,TT)
-x$Elo_j<-rep(SP,TT)
-x$Elo_pi_hat<-rep(NA,TT)
+x$Elo_i_before_match<-NA
+x$Elo_j_before_match<-NA
+x$Elo_i_after_match<-NA
+x$Elo_j_after_match<-NA
+x$Elo_pi_hat<-NA
 
 ################################# Weighted Elo
 
-x$WElo_i<-rep(SP,TT)
-x$WElo_j<-rep(SP,TT)
-x$WElo_pi_hat<-rep(NA,TT)
+x$WElo_i_before_match<-NA
+x$WElo_j_before_match<-NA
+x$WElo_i_after_match<-NA
+x$WElo_j_after_match<-NA
+x$WElo_pi_hat<-NA
 
 ################################# Standard errors for WElo
 
 if (CI==TRUE){
 
-x$WElo_i_lb<-rep(NA,TT)
-x$WElo_i_ub<-rep(NA,TT)
-x$WElo_j_lb<-rep(NA,TT)
-x$WElo_j_ub<-rep(NA,TT)
+x$WElo_i_lb<-NA
+x$WElo_i_ub<-NA
+x$WElo_j_lb<-NA
+x$WElo_j_ub<-NA
 
 ################################# Standard errors for Elo
 
-x$Elo_i_lb<-rep(NA,TT)
-x$Elo_i_ub<-rep(NA,TT)
-x$Elo_j_lb<-rep(NA,TT)
-x$Elo_j_ub<-rep(NA,TT)
+x$Elo_i_lb<-NA
+x$Elo_i_ub<-NA
+x$Elo_j_lb<-NA
+x$Elo_j_ub<-NA
 
 }
 
@@ -457,20 +465,30 @@ TT_new<-nrow(new_data)
 x_old<-x$dataset
 x_res<-x$results
 
-new_data$Elo_i<-SP    
-new_data$Elo_j<-SP
+new_data$Elo_i_before_match<-NA
+new_data$Elo_j_before_match<-NA
+new_data$Elo_i_after_match<-NA
+new_data$Elo_j_after_match<-NA
 new_data$Elo_pi_hat<-NA   
-new_data$WElo_i<-SP   
-new_data$WElo_j<-SP 
+
+new_data$WElo_i_before_match<-NA
+new_data$WElo_j_before_match<-NA
+new_data$WElo_i_after_match<-NA
+new_data$WElo_j_after_match<-NA
 new_data$WElo_pi_hat<-NA 
 
 x<-rbind(x_old,new_data)
 
-x$Elo_i[1:TT_old]<-x_res$Elo_i
-x$Elo_j[1:TT_old]<-x_res$Elo_j
+x$Elo_i_before_match[1:TT_old]<-x_res$Elo_i_before_match
+x$Elo_j_before_match[1:TT_old]<-x_res$Elo_j_before_match
+x$Elo_i_after_match[1:TT_old]<-x_res$Elo_i_after_match
+x$Elo_j_after_match[1:TT_old]<-x_res$Elo_j_after_match
 x$Elo_pi_hat[1:TT_old]<-x_res$Elo_pi_hat
-x$WElo_i[1:TT_old]<-x_res$WElo_i
-x$WElo_j[1:TT_old]<-x_res$WElo_j
+
+x$WElo_i_before_match[1:TT_old]<-x_res$WElo_i_before_match
+x$WElo_j_before_match[1:TT_old]<-x_res$WElo_j_before_match
+x$WElo_i_after_match[1:TT_old]<-x_res$WElo_i_after_match
+x$WElo_j_after_match[1:TT_old]<-x_res$WElo_j_after_match
 x$WElo_pi_hat[1:TT_old]<-x_res$WElo_pi_hat
 }
 
@@ -488,61 +506,84 @@ end_c<-nrow(x)
 
 for(tt in begin_c:end_c){
 
-i<-as.character(x$P_i[tt])
-j<-as.character(x$P_j[tt])
+player_i<-x$P_i[tt]
+player_j<-x$P_j[tt]
 
-### subset of matches played by player i and j, before the upcoming match
+#################################### check if player i has disputed a match before time tt
+if(match(player_i,x$P_i[1:tt])==tt & 
+is.na(match(player_i,x$P_j[1:tt]))
+   ) {
+x$Elo_i_before_match[tt]<-SP
+x$WElo_i_before_match[tt]<-SP
+N_i<-1
+} else {
+match_i_tt_1<-max(which(player_i==x$P_i[1:(tt-1)]),
+which(player_i==x$P_j[1:(tt-1)]))
 
-db_i<-subset(x,x$id<tt&(x$Winner==i|x$Loser==i))
-db_j<-subset(x,x$id<tt&(x$Winner==j|x$Loser==j))
+## number of matches played by i before match at time tt
+N_i<-length(c(which(player_i==x$P_i[1:(tt-1)]),
+which(player_i==x$P_j[1:(tt-1)])))
 
-N_i<-dim(db_i)[1]
-N_j<-dim(db_j)[1]
+x$Elo_i_before_match[tt]<-ifelse(
+player_i==x$P_i[match_i_tt_1],
+x$Elo_i_after_match[match_i_tt_1],
+x$Elo_j_after_match[match_i_tt_1])
+
+x$WElo_i_before_match[tt]<-ifelse(
+player_i==x$P_i[match_i_tt_1],
+x$WElo_i_after_match[match_i_tt_1],
+x$WElo_j_after_match[match_i_tt_1])
 
-if(N_i<=1){
-gs_i<-surf_i<-1
 }
 
-if(N_j<=1){
-gs_j<-surf_j<-1
+
+#################################### check if player j has disputed a match before time tt
+if(match(player_j,x$P_j[1:tt])==tt & 
+is.na(match(player_j,x$P_i[1:tt]))
+   ) {
+x$Elo_j_before_match[tt]<-SP
+x$WElo_j_before_match[tt]<-SP
+N_j<-1
+} else {
+match_j_tt_1<-max(which(player_j==x$P_i[1:(tt-1)]),
+which(player_j==x$P_j[1:(tt-1)]))
+
+## number of matches played by j before match at time tt
+N_j<-length(c(which(player_j==x$P_i[1:(tt-1)]),
+which(player_j==x$P_j[1:(tt-1)])))
+
+x$Elo_j_before_match[tt]<-ifelse(
+player_j==x$P_i[match_j_tt_1],
+x$Elo_i_after_match[match_j_tt_1],
+x$Elo_j_after_match[match_j_tt_1])
+
+x$WElo_j_before_match[tt]<-ifelse(
+player_j==x$P_i[match_j_tt_1],
+x$WElo_i_after_match[match_j_tt_1],
+x$WElo_j_after_match[match_j_tt_1])
+
 }
 
-if(N_i>=2){
-gs_i<-ifelse(db_i$Series[length(db_i$Series)]=="Grand Slam",1.1,1)
-}
+## if the match at time tt is a Grand Slam match
 
-if(N_j>=2){
-gs_j<-ifelse(db_j$Series[length(db_j$Series)]=="Grand Slam",1.1,1)
-}
+gs<-ifelse(x$Series[tt]=="Grand Slam",1.1,1)
 
 #### surface hard
 
-if(N_i>=2&K=="Surface_Hard"){
-surf_i<-ifelse(db_i$Surface[length(db_i$Surface)]=="Hard",1.1,1)
-}
-
-if(N_j>=2&K=="Surface_Hard"){
-surf_j<-ifelse(db_j$Surface[length(db_j$Surface)]=="Hard",1.1,1)
+if(K=="Surface_Hard"){
+surf<-ifelse(x$Surface[tt]=="Hard",1.1,1)
 }
 
 #### surface clay
 
-if(N_i>=2&K=="Surface_Clay"){
-surf_i<-ifelse(db_i$Surface[length(db_i$Surface)]=="Clay",1.1,1)
+if(K=="Surface_Clay"){
+surf<-ifelse(x$Surface[tt]=="Clay",1.1,1)
 }
 
-if(N_j>=2&K=="Surface_Clay"){
-surf_j<-ifelse(db_j$Surface[length(db_j$Surface)]=="Clay",1.1,1)
-}
+#### surface hard
 
-#### surface grass
-
-if(N_i>=2&K=="Surface_Grass"){
-surf_i<-ifelse(db_i$Surface[length(db_i$Surface)]=="Grass",1.1,1)
-}
-
-if(N_j>=2&K=="Surface_Grass"){
-surf_j<-ifelse(db_j$Surface[length(db_j$Surface)]=="Grass",1.1,1)
+if(K=="Surface_Grass"){
+surf<-ifelse(x$Surface[tt]=="Grass",1.1,1)
 }
 
 if(K=="Kovalchik"){
@@ -552,13 +593,13 @@ K_j<-250/(N_j+5)^0.4
 
 } else if (K=="Grand_Slam"){
 
-K_i<-250/(N_i+5)^0.4*gs_i
-K_j<-250/(N_j+5)^0.4*gs_j
+K_i<-250/(N_i+5)^0.4*gs
+K_j<-250/(N_j+5)^0.4*gs
 
 } else if (K=="Surface_Grass"|K=="Surface_Clay"|K=="Surface_Hard"){
 
-K_i<-250/(N_i+5)^0.4*surf_i
-K_j<-250/(N_j+5)^0.4*surf_j
+K_i<-250/(N_i+5)^0.4*surf
+K_j<-250/(N_j+5)^0.4*surf
 
 } else {
 
@@ -567,235 +608,77 @@ K_j<-K
 
 }
 
-if (N_i==0&N_j==0) { # "tt" is the first match for both i and j
-
 ############################# standard ELO
 
-x$Elo_pi_hat[tt]<-0.5
+x$Elo_pi_hat[tt]<-tennis_prob(x$Elo_i_before_match[tt],
+x$Elo_j_before_match[tt])
 
-x$Elo_i[tt]<-x$Elo_i[tt]+K_i*(x$Y_i[tt]-x$Elo_pi_hat[tt])
-x$Elo_j[tt]<-x$Elo_j[tt]+K_j*(x$Y_j[tt]-x$Elo_pi_hat[tt])
+x$Elo_i_after_match[tt]<-x$Elo_i_before_match[tt]+K_i*(
+x$Y_i[tt]-x$Elo_pi_hat[tt])
+
+x$Elo_j_after_match[tt]<-x$Elo_j_before_match[tt]+K_j*(
+x$Y_j[tt]-(1-x$Elo_pi_hat[tt]))
 
 ############################# Weighted ELO
 
-x$WElo_pi_hat[tt]<-0.5
+x$WElo_pi_hat[tt]<-tennis_prob(x$WElo_i_before_match[tt],
+x$WElo_j_before_match[tt])
 
 if(W=="GAMES"){
-x$WElo_i[tt]<-x$WElo_i[tt]+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_g_i[tt]
-x$WElo_j[tt]<-x$WElo_j[tt]+K_j*(x$Y_j[tt]-x$WElo_pi_hat[tt])*x$f_g_j[tt]
+
+x$WElo_i_after_match[tt]<-x$WElo_i_before_match[tt]+K_i*(
+x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_g_i[tt]
+
+x$WElo_j_after_match[tt]<-x$WElo_j_before_match[tt]+K_j*(
+x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_g_j[tt]
+
 } else {
-x$WElo_i[tt]<-x$WElo_i[tt]+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_s_i[tt]
-x$WElo_j[tt]<-x$WElo_j[tt]+K_j*(x$Y_j[tt]-x$WElo_pi_hat[tt])*x$f_s_j[tt]
+x$WElo_i_after_match[tt]<-x$WElo_i_before_match[tt]+K_i*(
+x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_s_i[tt]
+
+x$WElo_j_after_match[tt]<-x$WElo_j_before_match[tt]+K_j*(
+x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_s_j[tt]
 }
 
 ###################################################################
 
-} else if (N_i>=1&N_j==0){ # player i has played at least a match
-
-################################################################### standard Elo
-
-Elo_p_i<-ifelse(db_i[N_i,]$P_i==i,db_i[N_i,]$Elo_i,db_i[N_i,]$Elo_j)
-x$Elo_pi_hat[tt]<-tennis_prob(Elo_p_i,SP)
-
-x$Elo_i[tt]<-Elo_p_i+K_i*(x$Y_i[tt]-x$Elo_pi_hat[tt])
-x$Elo_j[tt]<-x$Elo_j[tt]+K_j*(x$Y_j[tt]-(1-x$Elo_pi_hat[tt]))
-
 ################################################################### Elo: Standard errors 
 
 if (CI==TRUE){
+
+############################# Elo
+
 p<-x$Elo_pi_hat[tt]
 q<-1-p
 
 ### Standard errors for i
 sim_i<-sample(0:1,B,replace=T,prob=c(q,p))
-x$Elo_i_lb[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),alpha/2)
-x$Elo_i_ub[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),1-alpha/2)
-
+x$Elo_i_lb[tt]<-stats::quantile(x$Elo_i_before_match[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),alpha/2)
+x$Elo_i_ub[tt]<-stats::quantile(x$Elo_i_before_match[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),1-alpha/2)
 
 ### Standard errors for j
 sim_j<-sample(0:1,B,replace=T,prob=c(p,q))
-x$Elo_j_lb[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),alpha/2)
-x$Elo_j_ub[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),1-alpha/2)
+x$Elo_j_lb[tt]<-stats::quantile(x$Elo_j_before_match[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),alpha/2)
+x$Elo_j_ub[tt]<-stats::quantile(x$Elo_j_before_match[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),1-alpha/2)
 
-}
+############################# WElo
 
-################################################################### Weighted ELO
-
-WElo_p_i<-ifelse(db_i[N_i,]$P_i==i,db_i[N_i,]$WElo_i,db_i[N_i,]$WElo_j)
-x$WElo_pi_hat[tt]<-tennis_prob(WElo_p_i,SP)
-
-if(W=="GAMES"){
-x$WElo_i[tt]<-WElo_p_i+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_g_i[tt]
-x$WElo_j[tt]<-x$WElo_j[tt]+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_g_j[tt]
-} else {
-x$WElo_i[tt]<-WElo_p_i+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_s_i[tt]
-x$WElo_j[tt]<-x$WElo_j[tt]+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_s_j[tt]
-}
-
-
-################################################################### WElo: Standard errors 
-
-if (CI==TRUE){
 p_w<-x$WElo_pi_hat[tt]
 q_w<-1-p_w
 
 ### Standard errors for i
 sim_i<-sample(0:1,B,replace=T,prob=c(q_w,p_w))
-x$WElo_i_lb[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),alpha/2)
-x$WElo_i_ub[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),1-alpha/2)
+x$WElo_i_lb[tt]<-stats::quantile(x$WElo_i_before_match[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),alpha/2)
+x$WElo_i_ub[tt]<-stats::quantile(x$WElo_i_before_match[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),1-alpha/2)
 
 
 ### Standard errors for j
 sim_j<-sample(0:1,B,replace=T,prob=c(p_w,q_w))
-x$WElo_j_lb[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),alpha/2)
-x$WElo_j_ub[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),1-alpha/2)
+x$WElo_j_lb[tt]<-stats::quantile(x$WElo_j_before_match[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),alpha/2)
+x$WElo_j_ub[tt]<-stats::quantile(x$WElo_j_before_match[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),1-alpha/2)
 
 }
 
-
-###################################################################
-
-} else if (N_i==0&N_j>=1){ # player j has played at a match
-
-################################################################### standard ELO
-
-Elo_p_j<-ifelse(db_j[N_j,]$P_i==j,db_j[N_j,]$Elo_i,db_j[N_j,]$Elo_j)
-x$Elo_pi_hat[tt]<-tennis_prob(SP,Elo_p_j)
-
-x$Elo_i[tt]<-x$Elo_i[tt]+K_i*(x$Y_i[tt]-x$Elo_pi_hat[tt])
-x$Elo_j[tt]<-Elo_p_j+K_j*(x$Y_j[tt]-(1-x$Elo_pi_hat[tt]))
-
-################################################################### Elo: Standard errors 
-
-if (CI==TRUE){
-p<-x$Elo_pi_hat[tt]
-q<-1-p
-
-### Standard errors for i
-sim_i<-sample(0:1,B,replace=T,prob=c(q,p))
-x$Elo_i_lb[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),alpha/2)
-x$Elo_i_ub[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),1-alpha/2)
-
-
-### Standard errors for j
-sim_j<-sample(0:1,B,replace=T,prob=c(p,q))
-x$Elo_j_lb[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),alpha/2)
-x$Elo_j_ub[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),1-alpha/2)
-
-}
-
-################################################################### Weighted ELO
-
-WElo_p_j<-ifelse(db_j[N_j,]$P_i==j,db_j[N_j,]$WElo_i,db_j[N_j,]$WElo_j)
-x$WElo_pi_hat[tt]<-tennis_prob(SP,WElo_p_j)
-
-if(W=="GAMES"){
-x$WElo_i[tt]<-x$WElo_i[tt]+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_g_i[tt]
-x$WElo_j[tt]<-WElo_p_j+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_g_j[tt]
-} else {
-x$WElo_i[tt]<-x$WElo_i[tt]+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_s_i[tt]
-x$WElo_j[tt]<-WElo_p_j+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_s_j[tt]
-}
-
-################################################################### WElo: Standard errors 
-
-if (CI==TRUE){
-p_w<-x$WElo_pi_hat[tt]
-q_w<-1-p_w
-
-### Standard errors for i
-sim_i<-sample(0:1,B,replace=T,prob=c(q_w,p_w))
-x$WElo_i_lb[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),alpha/2)
-x$WElo_i_ub[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),1-alpha/2)
-
-
-### Standard errors for j
-sim_j<-sample(0:1,B,replace=T,prob=c(p_w,q_w))
-x$WElo_j_lb[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),alpha/2)
-x$WElo_j_ub[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),1-alpha/2)
-
-}
-
-###################################################################
-
-} else if (N_i>=1&N_j>=1){ # both players have already played at least a match
-
-################################################################### standard ELO
-
-Elo_p_i<-ifelse(db_i[N_i,]$P_i==i,db_i[N_i,]$Elo_i,db_i[N_i,]$Elo_j)
-Elo_p_j<-ifelse(db_j[N_j,]$P_i==j,db_j[N_j,]$Elo_i,db_j[N_j,]$Elo_j)
-
-x$Elo_pi_hat[tt]<-tennis_prob(Elo_p_i,Elo_p_j)
-
-x$Elo_i[tt]<-Elo_p_i+K_i*(x$Y_i[tt]-x$Elo_pi_hat[tt])
-x$Elo_j[tt]<-Elo_p_j+K_j*(x$Y_j[tt]-(1-x$Elo_pi_hat[tt]))
-
-################################################################### Elo: Standard errors 
-
-if (CI==TRUE){
-p<-x$Elo_pi_hat[tt]
-q<-1-p
-
-### Standard errors for i
-sim_i<-sample(0:1,B,replace=T,prob=c(q,p))
-x$Elo_i_lb[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),alpha/2)
-x$Elo_i_ub[tt]<-stats::quantile(x$Elo_i[tt]+K_i*(sim_i-x$Elo_pi_hat[tt]),1-alpha/2)
-
-
-### Standard errors for j
-sim_j<-sample(0:1,B,replace=T,prob=c(p,q))
-x$Elo_j_lb[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),alpha/2)
-x$Elo_j_ub[tt]<-stats::quantile(x$Elo_j[tt]+K_j*(sim_j-(1-x$Elo_pi_hat[tt])),1-alpha/2)
-
-}
-
-################################################################### Weighted Elo
-
-WElo_p_i<-ifelse(db_i[N_i,]$P_i==i,db_i[N_i,]$WElo_i,db_i[N_i,]$WElo_j)
-WElo_p_j<-ifelse(db_j[N_j,]$P_i==j,db_j[N_j,]$WElo_i,db_j[N_j,]$WElo_j)
-
-x$WElo_pi_hat[tt]<-tennis_prob(WElo_p_i,WElo_p_j)
-
-if(W=="GAMES"){
-x$WElo_i[tt]<-WElo_p_i+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_g_i[tt]
-x$WElo_j[tt]<-WElo_p_j+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_g_j[tt]
-} else {
-x$WElo_i[tt]<-WElo_p_i+K_i*(x$Y_i[tt]-x$WElo_pi_hat[tt])*x$f_s_i[tt]
-x$WElo_j[tt]<-WElo_p_j+K_j*(x$Y_j[tt]-(1-x$WElo_pi_hat[tt]))*x$f_s_j[tt]
-}
-
-
-################################################################### WElo: Standard errors 
-
-if (CI==TRUE){
-p_w<-x$WElo_pi_hat[tt]
-q_w<-1-p_w
-
-### Standard errors for i
-sim_i<-sample(0:1,B,replace=T,prob=c(q_w,p_w))
-x$WElo_i_lb[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),alpha/2)
-x$WElo_i_ub[tt]<-stats::quantile(x$WElo_i[tt]+K_i*(sim_i-x$WElo_pi_hat[tt]),1-alpha/2)
-
-
-### Standard errors for j
-sim_j<-sample(0:1,B,replace=T,prob=c(p_w,q_w))
-x$WElo_j_lb[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),alpha/2)
-x$WElo_j_ub[tt]<-stats::quantile(x$WElo_j[tt]+K_j*(sim_j-(1-x$WElo_pi_hat[tt])),1-alpha/2)
-
-}
-
-###################################################################
-
-} else {
-
-x$Elo_i[tt]<- 9999
-x$Elo_j[tt]<- 9999
-
-x$WElo_i[tt]<- 9999
-x$WElo_j[tt]<- 9999
-
-}
-cat('match number', tt, 'of', nrow(x),'\n')
 } #end cycle
 
 cat('-----------------------------','\n') 
@@ -816,12 +699,16 @@ x_sub<-data.frame("Date"=x$Date,
 "P_j"=x$P_j,
 "Outcome_P_i"=x$Y_i,
 "Outcome_P_j"=x$Y_j,
-"Elo_i"=x$Elo_i,    
-"Elo_j"=x$Elo_j, 
+"Elo_i_before_match"=x$Elo_i_before_match,    
+"Elo_j_before_match"=x$Elo_j_before_match, 
 "Elo_pi_hat"=x$Elo_pi_hat,
-"WElo_i"=x$WElo_i,
-"WElo_j"=x$WElo_j,
+"Elo_i_after_match"=x$Elo_i_after_match,    
+"Elo_j_after_match"=x$Elo_j_after_match, 
+"WElo_i_before_match"=x$WElo_i_before_match,
+"WElo_j_before_match"=x$WElo_j_before_match,
 "WElo_pi_hat"=x$WElo_pi_hat,
+"WElo_i_after_match"=x$WElo_i_after_match,
+"WElo_j_after_match"=x$WElo_j_after_match,
 "Odds_i"=odds_i,
 "Odds_j"=odds_j,
 "Best_odds_i"=best_odds_i,
@@ -836,20 +723,24 @@ x_sub<-data.frame("Date"=x$Date,
 "P_j"=x$P_j,
 "Outcome_P_i"=x$Y_i,
 "Outcome_P_j"=x$Y_j,
-"Elo_i"=x$Elo_i,
-"Elo_i_lb"=x$Elo_i_lb, 
-"Elo_i_ub"=x$Elo_i_ub, 
-"Elo_j"=x$Elo_j, 
-"Elo_j_lb"=x$Elo_j_lb,
-"Elo_j_ub"=x$Elo_j_ub,   
+"Elo_i_before_match_lb"=x$Elo_i_lb, 
+"Elo_i_before_match"=x$Elo_i_before_match,
+"Elo_i_before_match_ub"=x$Elo_i_ub, 
+"Elo_j_before_match_lb"=x$Elo_j_lb,
+"Elo_j_before_match"=x$Elo_j_before_match, 
+"Elo_j_before_match_ub"=x$Elo_j_ub, 
 "Elo_pi_hat"=x$Elo_pi_hat,
-"WElo_i"=x$WElo_i,
-"WElo_i_lb"=x$WElo_i_lb,
-"WElo_i_ub"=x$WElo_i_ub,  
-"WElo_j"=x$WElo_j,
-"WElo_j_lb"=x$WElo_j_lb,
-"WElo_j_ub"=x$WElo_j_ub,  
+"Elo_i_after_match"=x$Elo_i_after_match,    
+"Elo_j_after_match"=x$Elo_j_after_match,
+"WElo_i_before_match_lb"=x$WElo_i_lb,
+"WElo_i_before_match"=x$WElo_i_before_match,
+"WElo_i_before_match_ub"=x$WElo_i_ub,  
+"WElo_j_before_match_lb"=x$WElo_j_lb,
+"WElo_j_before_match"=x$WElo_j_before_match,
+"WElo_j_before_match_ub"=x$WElo_j_ub,  
 "WElo_pi_hat"=x$WElo_pi_hat,
+"WElo_i_after_match"=x$WElo_i_after_match,
+"WElo_j_after_match"=x$WElo_j_after_match,
 "Odds_i"=odds_i,
 "Odds_j"=odds_j,
 "Best_odds_i"=best_odds_i,
@@ -880,10 +771,10 @@ results=x_sub,
 matches=paste("Number of matches:",nrow(x),sep=" "),
 period=paste("From",x_sub$Date[1],"to",x_sub$Date[nrow(x_sub)],sep=" "),
 loss=loss,
-highest_welo=paste("The player with the highest WElo rate, reached on", x_sub$Date[which.max(x_sub$WElo_i)], "is:",
-x_sub$P_i[which.max(x_sub$WElo_i)],sep=" "),
-highest_elo=paste("The player with the highest Elo rate, reached on", x_sub$Date[which.max(x_sub$Elo_i)], "is:",
-x_sub$P_i[which.max(x_sub$Elo_i)],sep=" "),
+highest_welo=paste("The player with the highest WElo rate, reached on", x_sub$Date[which.max(x_sub$WElo_i_after_match)], "is:",
+x_sub$P_i[which.max(x_sub$WElo_i_after_match)],sep=" "),
+highest_elo=paste("The player with the highest Elo rate, reached on", x_sub$Date[which.max(x_sub$Elo_i_after_match)], "is:",
+x_sub$P_i[which.max(x_sub$Elo_i_after_match)],sep=" "),
 dataset=x
 )
 
